@@ -1,6 +1,6 @@
 use numpy::{PyReadonlyArray1, PyReadonlyArray2, PyReadonlyArray3, ToPyArray};
 use std::fmt::{Display, Write};
-use std::ops::{Add, Range};
+use std::ops::{Add, Mul, Range};
 
 use pyo3::prelude::*;
 
@@ -12,6 +12,28 @@ pub struct RaggedBuffer<T> {
     items: usize,
 }
 
+pub trait BinOp<T> {
+    fn op(lhs: T, rhs: T) -> T;
+}
+
+pub struct BinOpAdd;
+
+impl<T: Add<T, Output = T>> BinOp<T> for BinOpAdd {
+    #[inline]
+    fn op(lhs: T, rhs: T) -> T {
+        lhs + rhs
+    }
+}
+
+pub struct BinOpMul;
+
+impl<T: Mul<T, Output = T>> BinOp<T> for BinOpMul {
+    #[inline]
+    fn op(lhs: T, rhs: T) -> T {
+        lhs * rhs
+    }
+}
+
 impl<T: numpy::Element + Copy + Display + Add<Output = T> + std::fmt::Debug> RaggedBuffer<T> {
     pub fn new(features: usize) -> Self {
         RaggedBuffer {
@@ -71,6 +93,7 @@ impl<T: numpy::Element + Copy + Display + Add<Output = T> + std::fmt::Debug> Rag
     pub fn clear(&mut self) {
         self.data.clear();
         self.subarrays.clear();
+        self.items = 0;
     }
 
     pub fn as_array<'a>(
@@ -79,7 +102,7 @@ impl<T: numpy::Element + Copy + Display + Add<Output = T> + std::fmt::Debug> Rag
     ) -> &'a numpy::PyArray<T, numpy::ndarray::Dim<[usize; 2]>> {
         self.data
             .to_pyarray(py)
-            .reshape((self.data.len() / self.features, self.features))
+            .reshape((self.items, self.features))
             .unwrap()
     }
 
@@ -211,31 +234,11 @@ impl<T: numpy::Element + Copy + Display + Add<Output = T> + std::fmt::Debug> Rag
         Ok(array)
     }
 
-    pub fn add(&self, rhs: &RaggedBuffer<T>) -> PyResult<RaggedBuffer<T>> {
+    pub fn binop<Op: BinOp<T>>(&self, rhs: &RaggedBuffer<T>) -> PyResult<RaggedBuffer<T>> {
         if self.features == rhs.features && self.subarrays == rhs.subarrays {
             let mut data = Vec::with_capacity(self.data.len());
             for i in 0..self.data.len() {
-                data.push(self.data[i] + rhs.data[i]);
-            }
-            Ok(RaggedBuffer {
-                data,
-                subarrays: self.subarrays.clone(),
-                features: self.features,
-                items: self.items,
-            })
-        } else if self.features == rhs.features
-            && self.subarrays.len() == rhs.subarrays.len()
-            && self.subarrays.iter().all(|r| r.end - r.start == 1)
-        {
-            let mut data = Vec::with_capacity(self.data.len());
-            for (subarray, rhs_subarray) in self.subarrays.iter().zip(rhs.subarrays.iter()) {
-                let mut i = subarray.start * self.features;
-                while i < subarray.end * self.features {
-                    for j in rhs_subarray.clone() {
-                        data.push(self.data[i] + rhs.data[j]);
-                    }
-                    i += self.features;
-                }
+                data.push(Op::op(self.data[i], rhs.data[i]));
             }
             Ok(RaggedBuffer {
                 data,
@@ -247,7 +250,27 @@ impl<T: numpy::Element + Copy + Display + Add<Output = T> + std::fmt::Debug> Rag
             && self.subarrays.len() == rhs.subarrays.len()
             && rhs.subarrays.iter().all(|r| r.end - r.start == 1)
         {
-            rhs.add(self)
+            let mut data = Vec::with_capacity(self.data.len());
+            for (subarray, rhs_subarray) in self.subarrays.iter().zip(rhs.subarrays.iter()) {
+                for item in subarray.clone() {
+                    let lhs_offset = item * self.features;
+                    let rhs_offset = rhs_subarray.start * self.features;
+                    for i in 0..self.features {
+                        data.push(Op::op(self.data[lhs_offset + i], rhs.data[rhs_offset + i]));
+                    }
+                }
+            }
+            Ok(RaggedBuffer {
+                data,
+                subarrays: self.subarrays.clone(),
+                features: self.features,
+                items: self.items,
+            })
+        } else if self.features == rhs.features
+            && self.subarrays.len() == rhs.subarrays.len()
+            && self.subarrays.iter().all(|r| r.end - r.start == 1)
+        {
+            rhs.binop::<Op>(self)
         } else {
             Err(pyo3::exceptions::PyValueError::new_err(format!(
                 "Dimensions mismatch: ({}, {:?}, {}) != ({}, {:?}, {})",
@@ -267,9 +290,9 @@ impl<T: numpy::Element + Copy + Display + Add<Output = T> + std::fmt::Debug> Rag
         }
     }
 
-    pub fn add_scalar(&self, scalar: T) -> RaggedBuffer<T> {
+    pub fn op_scalar<Op: BinOp<T>>(&self, scalar: T) -> RaggedBuffer<T> {
         RaggedBuffer {
-            data: self.data.iter().map(|x| *x + scalar).collect(),
+            data: self.data.iter().map(|x| Op::op(*x, scalar)).collect(),
             subarrays: self.subarrays.clone(),
             features: self.features,
             items: self.items,
@@ -292,9 +315,20 @@ impl<T: numpy::Element + Copy + Display + Add<Output = T> + std::fmt::Debug> Rag
                     items: self.items,
                 })
             }
-            1 => Err(pyo3::exceptions::PyValueError::new_err(
-                "Sequence indices not yet implemented",
-            )),
+            1 => {
+                let mut indices = Vec::with_capacity(self.items);
+                for subarray in &self.subarrays {
+                    for (i, _) in subarray.clone().enumerate() {
+                        indices.push(i as i64);
+                    }
+                }
+                Ok(RaggedBuffer {
+                    subarrays: self.subarrays.clone(),
+                    data: indices,
+                    features: 1,
+                    items: self.items,
+                })
+            }
             _ => Err(pyo3::exceptions::PyValueError::new_err(format!(
                 "Invalid dimension {}",
                 dim
